use super::{Error, Sector, SectorWriter};
use cdtypes::types::time::Time;
use std::io::Write;

pub struct BinCueWriter {
    bin_out:      std::boxed::Box<dyn Write>,
    cd_time:      Time,   
}

impl BinCueWriter {
    pub fn new<T: Write + 'static>(writer: T) -> BinCueWriter {
        BinCueWriter{bin_out: std::boxed::Box::new(writer), cd_time: Time::cd_start()}
    }

    fn finalize(&mut self, sector: &mut Sector) -> Result<(), Error> {
        sector.finalize();

        if let Some(header) = sector.get_header_mut() {
            header.set_time(self.cd_time.clone());
        }

        self.cd_time.add_sector()?;
        Ok(())
    }
}

impl SectorWriter for BinCueWriter {
    fn write(&mut self, mut sector: Sector) -> Result<usize, Error> {
        self.finalize(&mut sector)?;

        Ok(self.bin_out.write(sector.as_raw())?)
    }
}